import xbmcplugin

from resources.lib.api.api import API
from resources.lib.const import ROUTE, explicit_genres, MEDIA_TYPE, COMMAND, DIR_TYPE, SETTINGS, RENDER_TYPE
from resources.lib.gui import MainMenuFolderItem, DirectoryItem, MediaItem, ParentItem
from resources.lib.gui.renderers.directory_renderer import DirectoryRenderer
from resources.lib.gui.renderers.media_info_renderer import MediaInfoRenderer
from resources.lib.kodilogging import logger
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.utils import first
from resources.lib.utils.kodiutils import get_string

has_children_map = {
    False: MediaItem,
    True: ParentItem
}


class MediaListRenderer:
    def __init__(self):
        """
        :param callable on_stream_selected: Called when a stream is selected.
        """
        self.media_has_children_map = {
            True: lambda media: router.get_url_api(ROUTE.GET_MEDIA, API.FILTER.parent(media.get('_id'))),
            False: lambda media: MediaListRenderer._get_stream_url(media),
        }

    def __call__(self, handle, media_list_gui, pagination, render_type=RENDER_TYPE.DEFAULT):
        logger.debug('Renderer %s call' % self)
        MediaListRenderer.add_navigation(media_list_gui, bottom=pagination)
        if pagination:
            MediaListRenderer.add_pagination(media_list_gui, pagination, render_type)
        MediaListRenderer.render(handle, DIR_TYPE.MOVIES, [item.build() for item in media_list_gui])

    def __repr__(self):
        return self.__class__.__name__

    @staticmethod
    def _get_stream_url(media):
        api_url = API.URL.streams(media.get('_id'))
        source = API.get_source(media)
        return router.get_url(ROUTE.GET_STREAMS,
                              url=api_url,
                              media_id=media.get('_id'),
                              root_parent_id=source.get('root_parent', media.get('_id')))

    def build_media_list(self, media_list, title_builder, *args):
        media_list_gui = []
        if not settings.as_bool(SETTINGS.EXPLICIT_CONTENT):
            MediaListRenderer.explicit_filter(media_list)
        for media in media_list.get('data'):
            source = API.get_source(media)
            info_labels, art = MediaInfoRenderer.merge_info_labels(source)
            title = title_builder(source, *args) or info_labels.get('originaltitle')
            cast = source.get('cast')
            has_children = source.get('children_count', 0) > 0
            has_streams = MediaInfoRenderer.stream_available(source)
            # if not has_children and not has_streams:
            #     continue
            url = self.media_has_children_map[has_children](media)
            gui_item = MediaListRenderer.build_media_item_gui(has_children_map[has_children], source, url,
                                                              title, has_streams, art, cast,
                                                              info_labels)
            media_list_gui.append(gui_item)

        return media_list_gui

    @staticmethod
    def render(handle, dir_type, list_items):
        # as_type=None causes all items to show their icons and spam bullshit in log
        with DirectoryRenderer.start_directory(handle, as_type=dir_type):
            xbmcplugin.addDirectoryItems(handle, list_items)

    @staticmethod
    def add_navigation(list_items, bottom=False):
        if settings.as_bool(SETTINGS.SHOW_TOP_NAVIGATION):
            item = MainMenuFolderItem(url=router.get_url_command(COMMAND.CLEAR_PATH))
            list_items.insert(0, item)
        if bottom:
            item = MainMenuFolderItem(url=router.get_url_command(COMMAND.CLEAR_PATH))
            list_items.append(item)

    @staticmethod
    def add_pagination(list_items, pagination, render_type):
        item = MediaListRenderer.next_page_item(pagination, render_type)
        if settings.as_bool(SETTINGS.SHOW_TOP_NAVIGATION):
            list_items.insert(0, item)
        list_items.append(item)

    @staticmethod
    def next_page_item(pagination, render_type):
        return DirectoryItem(
            title=MediaListRenderer.next_page_title(pagination['page'] + 1, pagination['pageCount']),
            url=router.get_url(ROUTE.GET_MEDIA, render_type=render_type, url=pagination['next']))

    @staticmethod
    def next_page_title(page, page_count):
        return '{} ({}/{})'.format(get_string(30203), page, page_count)

    @staticmethod
    def build_media_item_gui(item_type, source, url, title, playable, art=None, cast=None, info_labels=None):
        # STRINGS.TITLE_YEAR.format(title=info_labels.get('title').encode('utf-8'), year=info_labels.get('year')
        return item_type(
            title=title,
            url=url,
            art=art,
            cast=cast,
            playable=playable,
            info_labels=info_labels,
            stream_info=source.get('stream_info'),
            services=source.get('services')
        )

    @staticmethod
    def explicit_filter(media_list):
        filtered_list = []
        for media in media_list.get('data'):
            source = API.get_source(media)
            genres = source.get('info_labels').get('genre')
            is_blocked = bool(set(genres).intersection(explicit_genres))
            if is_blocked:
                continue
            filtered_list.append(media)
        media_list.update({'data': filtered_list})
